clear
close all
clc

% fx_uz_SNR.m
% This program demonstrates the SNR improvement by nyquist rate, oversampling and delta
% sigma modulator for a one bit input signal.
% SNR for Oversampling: SNR=6.02w+20log10(L), L=2^x, x=0,...10
% SNR for DSM: SNR=6.02w+10log10(n(L^n)pi^(1-n)), n=2od+1: Derived from
% variances
% Input arguments:
%   1.  w:   word length
%   2.  od:     Order of DSM
%   3.  L:     Oversampling factor
%   4.  x:     power of L
% Output arguments:
%   1.  SNR:    Signal to noise ratio
%   2.  H:      Magnitude response

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Reference:
% [1]:  U.Zölzer, Digital Audio signal processing 3rd Edition (chap.4). John Wiley & Sons, 2022 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
w=1;
x=1:14;                                               % power of oversampling factor
od1=1; od2=2;od3=3;                                   % different orders
n1=2*od1+1;n2=2*od2+1;n3=2*od3+1;
SNR=6.02*w*ones(1,length(x));
SNR0=6.02*w+10*log10(2.^x);
SNR1=6.02*w+10*log10(n1*(2.^(x*n1))*pi^(1-n1)+eps);
SNR2=6.02*w+10*log10(n2*(2.^(x*n2))*pi^(1-n2)+eps);
SNR3=6.02*w+10*log10(n2*(2.^(x*n3))*pi^(1-n3)+eps);
N=1024;
k=(0:N-1)/N; % normalized frequency (f/fs)
k=k(1:N/2); 
Omega=2*pi*k; 
H1=(abs(1-exp(-j*Omega))).^(2*od1);
H2=(abs(1-exp(-j*Omega))).^(2*od2);
H3=(abs(1-exp(-j*Omega))).^(2*od3);

% plots
figure(1)
plot(x,SNR,x,SNR0,'r', x,SNR1,'g',x,SNR2,'k',x,SNR3,'m'); 
axis([2 12 0 140]); xlabel('(L=2^{x})  x \rightarrow');
ylabel('SNR [dB] \rightarrow');
 legend('Nyquist Rate','Oversampling','DSM first order','DSM second order','DSM third order')
title ('Improvement of SNR')
grid on
